@@ -5,7 +5,11 @@ import porespy as ps
 from numba import njit
 import scipy.spatial as sptl
 import scipy.ndimage as spim
+import scipy.stats as spst
+from deprecated import deprecated
 from porespy.tools import norm_to_uniform, ps_ball, ps_disk, get_border
+from porespy.tools import extract_subsection
+from porespy.tools import insert_sphere
 from porespy import settings
 from typing import List
 from loguru import logger
@@ -51,7 +55,7 @@ def insert_shape(im, element, center=None, corner=None, value=1, mode="overwrite
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/insert_shape.html>`_
+    <https://porespy.org/examples/generators/reference/insert_shape.html>`_
     to view online example.
 
     """
@@ -97,7 +101,12 @@ def insert_shape(im, element, center=None, corner=None, value=1, mode="overwrite
     return im
 
 
-def RSA(im_or_shape: np.array,
+@deprecated("This function has been renamed to rsa (lowercase to meet pep8")
+def RSA(*args, **kwargs):
+    return rsa(*args, **kwargs)
+
+
+def rsa(im_or_shape: np.array,
         r: int,
         volume_fraction: int = 1,
         clearance: int = 0,
@@ -177,7 +186,7 @@ def RSA(im_or_shape: np.array,
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/RSA.html>`_
+    <https://porespy.org/examples/generators/reference/RSA.html>`_
     to view online example.
 
     """
@@ -317,7 +326,7 @@ def _make_choice(options_im, free_sites):
     return coords, count
 
 
-def bundle_of_tubes(shape: List[int], spacing: int):
+def bundle_of_tubes(shape: List[int], spacing: int, distribution=None, smooth=True):
     r"""
     Create a 3D image of a bundle of tubes, in the form of a rectangular
     plate with randomly sized holes through it.
@@ -330,7 +339,9 @@ def bundle_of_tubes(shape: List[int], spacing: int):
         1 voxel is assumed.
     spacing : int
         The center to center distance of the holes.  The hole sizes will
-        be randomly distributed between this values down to 3 voxels.
+        be distributed between this values down to 3 voxels.
+    distribution : scipy.stats object
+        A handle to a scipy stats object with the desired parameters.
 
     Returns
     -------
@@ -340,42 +351,42 @@ def bundle_of_tubes(shape: List[int], spacing: int):
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/bundle_of_tubes.html>`_
+    <https://porespy.org/examples/generators/reference/bundle_of_tubes.html>`_
     to view online example.
 
     """
     shape = np.array(shape)
-    if np.size(shape) == 1:
-        shape = np.full((3,), int(shape))
-    if np.size(shape) == 2:
+    if len(shape) == 2:
         shape = np.hstack((shape, [1]))
-    temp = np.zeros(shape=shape[:2])
-    Xi = np.ceil(
-        np.linspace(spacing / 2, shape[0] - (spacing / 2) - 1, int(shape[0] / spacing))
-    )
-    Xi = np.array(Xi, dtype=int)
-    Yi = np.ceil(
-        np.linspace(spacing / 2, shape[1] - (spacing / 2) - 1, int(shape[1] / spacing))
-    )
-    Yi = np.array(Yi, dtype=int)
-    temp[tuple(np.meshgrid(Xi, Yi))] = 1
-    inds = np.where(temp)
+    shape2 = shape[shape > 1]
+    im = ~lattice_spheres(shape=shape2,
+                          r=1,
+                          offset=0.5*spacing,
+                          spacing=spacing,
+                          lattice='sc')
+    N = im.sum()
+    if distribution is None:
+        # +1 below is because randint 4.X gives a max of 3
+        distribution = spst.randint(low=3, high=spacing/2 + 1)
+        Rs = distribution.rvs(N)
+    else:
+        Rs = distribution.rvs(N)
+        Rs = np.around(np.clip(Rs, a_min=1, a_max=spacing/2), decimals=0).astype(int)
+    temp = np.zeros_like(im)
+    inds = np.where(im)
     for i in range(len(inds[0])):
-        r = np.random.randint(1, (spacing / 2))
-        try:
-            s1 = slice(inds[0][i] - r, inds[0][i] + r + 1)
-            s2 = slice(inds[1][i] - r, inds[1][i] + r + 1)
-            temp[s1, s2] = ps_disk(r)
-        except ValueError:
-            odd_shape = np.shape(temp[s1, s2])
-            temp[s1, s2] = ps_disk(r)[: odd_shape[0], : odd_shape[1]]
-    im = np.broadcast_to(array=np.atleast_3d(temp), shape=shape)
-    return im
+        c = np.hstack([j[i] for j in inds])
+        temp = insert_sphere(im=temp, c=c, r=Rs[i])
+    # Add 3rd dimension back
+    temp = np.tile(np.atleast_3d(temp), [1, 1, shape[2]])
+    return temp
 
 
-def polydisperse_spheres(
-    shape: List[int], porosity: float, dist, nbins: int = 5, r_min: int = 5
-):
+def polydisperse_spheres(shape: List[int],
+                         porosity: float,
+                         dist,
+                         nbins: int = 5,
+                         r_min: int = 5):
     r"""
     Create an image of randomly placed, overlapping spheres with a
     distribution of radii.
@@ -412,7 +423,7 @@ def polydisperse_spheres(
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/polydisperse_spheres.html>`_
+    <https://porespy.org/examples/generators/reference/polydisperse_spheres.html>`_
     to view online example.
 
     """
@@ -433,20 +444,22 @@ def polydisperse_spheres(
     return im
 
 
-def voronoi_edges(shape: List[int], r: int, ncells: int, flat_faces: bool = True):
+def voronoi_edges(shape: List[int], ncells: int, r: int = 0,
+                  flat_faces: bool = True):
     r"""
     Create an image from the edges of a Voronoi tessellation.
 
     Parameters
     ----------
     shape : array_like
-        The size of the image to generate in [Nx, Ny, Nz] where Ni is the
-        number of voxels in each direction.
-    radius : int
-        The radius to which Voronoi edges should be dilated in the final
-        image.
+        The size of the image to generate in [Nx, Ny, <Nz>] where Ni is the
+        number of voxels in each direction.  If Nz is not given a 2D image
+        is returned.
     ncells : int
         The number of Voronoi cells to include in the tesselation.
+    radius : int, optional
+        The radius to which Voronoi edges should be dilated in the final
+        image.  If not given then edges are a single pixel/voxel thick.
     flat_faces : bool
         Whether the Voronoi edges should lie on the boundary of the
         image (``True``), or if edges outside the image should be removed
@@ -460,7 +473,7 @@ def voronoi_edges(shape: List[int], r: int, ncells: int, flat_faces: bool = True
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/voronoi_edges.html>`_
+    <https://porespy.org/examples/generators/reference/voronoi_edges.html>`_
     to view online example.
 
     """
@@ -469,26 +482,26 @@ def voronoi_edges(shape: List[int], r: int, ncells: int, flat_faces: bool = True
     if np.size(shape) == 1:
         shape = np.full((3,), int(shape))
     im = np.zeros(shape, dtype=bool)
-    base_pts = np.random.rand(ncells, 3) * shape
+    base_pts = tuple(np.around(np.random.rand(ncells, im.ndim) * (shape-1),
+                               decimals=0).T.astype(int))
+    im[tuple(base_pts)] = True
+    pw = [(s, s) for s in im.shape]
     if flat_faces:
-        # Reflect base points
-        Nx, Ny, Nz = shape
-        orig_pts = base_pts
-        base_pts = np.vstack((base_pts, [-1, 1, 1] * orig_pts + [2.0 * Nx, 0, 0]))
-        base_pts = np.vstack((base_pts, [1, -1, 1] * orig_pts + [0, 2.0 * Ny, 0]))
-        base_pts = np.vstack((base_pts, [1, 1, -1] * orig_pts + [0, 0, 2.0 * Nz]))
-        base_pts = np.vstack((base_pts, [-1, 1, 1] * orig_pts))
-        base_pts = np.vstack((base_pts, [1, -1, 1] * orig_pts))
-        base_pts = np.vstack((base_pts, [1, 1, -1] * orig_pts))
-    vor = sptl.Voronoi(points=base_pts)
+        im = np.pad(im, pad_width=pw, mode='symmetric')
+    else:
+        im = np.pad(im, pad_width=pw, mode='constant', constant_values=0)
+    base_pts = np.where(im)
+    vor = sptl.Voronoi(points=np.vstack(base_pts).T)
     vor.vertices = np.around(vor.vertices)
     vor.vertices *= (np.array(im.shape) - 1) / np.array(im.shape)
     vor.edges = _get_Voronoi_edges(vor)
+    im = np.zeros_like(im)
     for row in vor.edges:
-        pts = vor.vertices[row].astype(int)
+        pts = np.around(vor.vertices[row], decimals=0).astype(int)
         if np.all(pts >= 0) and np.all(pts < im.shape):
             line_pts = line_segment(pts[0], pts[1])
             im[tuple(line_pts)] = True
+    im = extract_subsection(im=im, shape=shape)
     im = edt(~im) > r
     return im
 
@@ -562,12 +575,14 @@ def lattice_spheres(shape: List[int],
     lattice : str
         Specifies the type of lattice to create. Options are:
 
-        'sc'
-            Simple Cubic (default)
-        'fcc'
-            Face Centered Cubic
-        'bcc'
-            Body Centered Cubic
+        ======= ===============================================================
+        option  description
+        ======= ===============================================================
+        'sc'    Simple cubic (default), works in both 2D and 3D.
+        'tri'   Triangular, only works in 2D
+        'fcc'   Face centered cubic, only works in 3D
+        'bcc'   Body centered cubic, only works on 3D
+        ======= ===============================================================
 
     Returns
     -------
@@ -582,7 +597,7 @@ def lattice_spheres(shape: List[int],
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/lattice_spheres.html>`_
+    <https://porespy.org/examples/generators/reference/lattice_spheres.html>`_
     to view online example.
 
     """
@@ -612,12 +627,16 @@ def lattice_spheres(shape: List[int],
     # Parse offset and spacing args
     if spacing is None:
         spacing = 2*r
-    if isinstance(spacing, int):
+    if isinstance(spacing, (int, float)):
+        spacing = int(spacing)
         spacing = [spacing]*im.ndim
+    spacing = np.array(spacing, dtype=int)
     if offset is None:
         offset = r
-    if isinstance(offset, int):
+    if isinstance(offset, (int, float)):
+        offset = int(offset)
         offset = [offset]*im.ndim
+    offset = np.array(offset, dtype=int)
 
     if lattice == 'sq':
         im[offset[0]::spacing[0],
@@ -698,7 +717,7 @@ def overlapping_spheres(shape: List[int],
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/overlapping_spheres.html>`_
+    <https://porespy.org/examples/generators/reference/overlapping_spheres.html>`_
     to view online example.
 
     """
@@ -796,7 +815,7 @@ def blobs(shape: List[int], porosity: float = 0.5, blobiness: int = 1,
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/blobs.html>`_
+    <https://porespy.org/examples/generators/reference/blobs.html>`_
     to view online example.
 
     """
@@ -1012,7 +1031,7 @@ def cylinders(shape: List[int],
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/cylinders.html>`_
+    <https://porespy.org/examples/generators/reference/cylinders.html>`_
     to view online example.
 
     """
@@ -1096,7 +1115,7 @@ def line_segment(X0, X1):
     Examples
     --------
     `Click here
-    <https://porespy.org/examples/generators/howtos/line_segment.html>`_
+    <https://porespy.org/examples/generators/reference/line_segment.html>`_
     to view online example.
 
     """
